import java.io.*;
import java.util.*;

public class BisSNPSummarize {
	private static String[] input = null;
/*	{
		"SJNORM010331_G24-HumanCD34-Set4-Day0_",
		"SJNORM010331_G25-HumanCD34-Set4-Day3_",
		"SJNORM010331_G26-HumanCD34-Set4-Day12_",
		"SJNORM010331_G27-HumanCD34-Set5-Day0_",
		"SJNORM010331_G28-HumanCD34-Set5-Day3_",
		"SJNORM010331_G29-HumanCD34-Set5-Day12_",
		"SJNORM010331_G30-HumanCD34-Set6-Day0_",
		"SJNORM010331_G31-HumanCD34-Set6-Day3_",
		"SJNORM010331_G32-HumanCD34-Set6-Day12_"
	};*/
	private static String output = null; //"SJNORM_set456_";
	private static String[] tag = null; //{"G24", "G25", "G26", "G27", "G28", "G29", "G30", "G31", "G32"};
	
	public static void main(String[] args) {
		int size = 0;
		BisCpG head = null, current = null, previous = null;
		String chr = null, line = null, currentTag = null;
		boolean withPoor = false, debug = args.length > 2, count=false;
		int thresh = 5; //coverage threshold
		try {
			BufferedReader br = new BufferedReader(new FileReader(args[0]));
			boolean inputRead = false, outputRead = false, tagRead = false;
			while ((line = br.readLine()) != null) {
				if (line.startsWith("[PREFIX]")) {
					inputRead = true; outputRead = false; tagRead = false;
					input = new String[(size > 0) ? size : 10];
					size = 0;
				} else if (line.startsWith("[/PREFIX]")) {
					inputRead = false;
					if (size == 0) {
						input = null;
					} else if (input.length > size) {
						String[] stmp = new String[size];
						System.arraycopy(input, 0, stmp, 0, size);
						input = stmp;
					}
					//System.out.println("Predix length: " + input.length);
				} else if (line.startsWith("[OUTPUT]")) {
					outputRead = true; inputRead = false; tagRead = false;
				} else if (line.startsWith("[WITHPOOR]")) {
					withPoor = true;
                                } else if (line.startsWith("[COUNTS]")) {
                                        count = true;
				} else if (line.startsWith("[THRESH]")) {
					StringTokenizer st = new StringTokenizer(line, "\t ");
					st.nextToken();
					thresh = Integer.parseInt(st.nextToken());
				} else if (line.startsWith("[/OUTPUT]")) {
					outputRead = false;
				} else if (line.startsWith("[TAG]")) {
					inputRead = false; outputRead = false; tagRead = true;
					tag = new String[(size > 0) ? size : 10];
					size = 0;
				} else if (line.startsWith("[/TAG]")) {
					tagRead = false;
					if (size == 0) {
						tag = null;
					} else if (tag.length > size) {
						String[] stmp = new String[size];
						System.arraycopy(tag, 0, stmp, 0, size);
						tag = stmp;
					}
					//System.out.println("Tag length: " + tag.length);
				} else if (inputRead) {
					if (size == input.length) {
						String[] stmp = new String[2 * size];
						System.arraycopy(input, 0, stmp, 0, size);
						input = stmp;
					}
					input[size++] = line;
				} else if (tagRead) {
					if (size == tag.length) {
						String[] stmp = new String[2 * size];
						System.arraycopy(tag, 0, stmp, 0, size);
						tag = stmp;
					}
					tag[size++] = line;
				} else if (outputRead) {
					output = line;
				}
			}
			br.close();
			if (tag == null) {
				tag = input;
			} else if (tag.length != input.length) {
				System.err.println("Error in config file " + args[0] + ": different length in Prefix (" + input.length + ") and Tag (" + tag.length + ").");
				System.exit(-10);
			}
			
			if (thresh > 5) {
			    if (count) {
			    	output += (args[1] + "_Countsummary_" + thresh + ".txt");
			    } else {
				output += (args[1] + "_summary_" + thresh + ".txt");
			    }
			} else {
			    if (count) {
			    	output += (args[1] + "_Countsummary.txt");
			    } else {
				output += (args[1] + "_summary.txt");
			    }
			}
			
			int total = 0;
			for (int i = 0; i < size; i++) {
				System.out.println("Reading " + input[i] + args[1] + "_meth_good.txt.");
				currentTag = tag[i];
				br = new BufferedReader(new FileReader(input[i] + args[1] + "_meth_good.txt"));
				line = br.readLine();
				current = head;
				previous = null;
				int lineCounts = 1;
				while ((line = br.readLine()) != null) {
					StringTokenizer st = new StringTokenizer(line, "\t");
					lineCounts++;
					String s = st.nextToken();
					st.nextToken();
					int pos = Integer.parseInt(st.nextToken());
					if (debug && pos == 3291740) System.out.println("Tag " + tag[i] + " Lines " + lineCounts + ": " + line);
					double beta = Double.parseDouble(st.nextToken());
					double mval = Double.parseDouble(st.nextToken());
					if (count) {
					    mval = Double.parseDouble(st.nextToken()); 
					} else {
					    st.nextToken();
					}
					double cov = Double.parseDouble(st.nextToken());
					if (cov < thresh) continue;
					if (count) beta = cov - mval;
					int abo = Integer.parseInt(st.nextToken());
					int est = Integer.parseInt(st.nextToken());

					if (chr == null) {
						chr = s;
						current = new BisCpG(pos, size);
						head = current;
						continue;
					}
					if (debug && pos == 3291740) System.out.println("Starting while");
					if (previous != null && previous.getPos() > pos) {
						System.err.println("Incorrect line order in " + input[i] + args[1] + "_meth_poor.txt (#" + lineCounts + "): " + line);
						current = head;
					}
					while (current != null && current.getPos() < pos) {
						if (debug && pos == 3291740 && current != null) System.out.println(current.getPos() + "\t" + pos);
						if (debug && pos == 3291740) System.out.print(current.getPos() + " ");
						previous = current;
						current = current.getNext();
					//	if (current != null && current.getPos() < previous.getPos()) {
					//		previous.setNext(null);
					//		current = null;
					//		break;
					//	}
					}
					if (debug && pos == 3291740) {
						System.out.println();
						System.out.println("Starting if");
						System.out.println("Previous: " + previous.getPos() + "; current: " + current.getPos() + "; next: " + current.getNext().getPos());
					}
					if (current == null || current.getPos() > pos) {
						BisCpG bc = new BisCpG(pos, size);
						total++;
						if (previous == null) {
							head = bc;
						} else {
							previous.setNext(bc);
						}
						bc.setNext(current);
						current = bc;
					}
					current.setCpG(i, beta, mval, abo, est);
					if (debug && pos == 3291740) {
						System.out.println("Done");
						System.out.println("Previous: " + previous.getPos() + "; current: " + current.getPos() + "; next: " + current.getNext().getPos());
					}
					
				}
				br.close();
				//System.out.println("Total lines read: " + lineCounts);
				//System.out.println("Total CpGs read: " + total);
				//current = head;
				//int cur = 0;
				//while (current != null) {
				//	current = current.getNext();
				//	cur++;
				//}
				//System.out.println("Total CpGs in queue: " + cur);
				if (!withPoor) continue;

				System.out.println("Reading " + input[i] + args[1] + "_meth_poor.txt.");
				br = new BufferedReader(new FileReader(input[i] + args[1] + "_meth_poor.txt"));
				line = br.readLine();
				current = head;
				previous = null;
				lineCounts = 1;
				while ((line = br.readLine()) != null) {
					StringTokenizer st = new StringTokenizer(line, "\t");
					lineCounts++;
					String s = st.nextToken();
					st.nextToken();
					int pos = Integer.parseInt(st.nextToken());
					if (debug && pos == 3291740) System.out.println("Tag " + tag[i] + " Lines " + lineCounts + ": " + line);
					double beta = Double.parseDouble(st.nextToken());
					double mval = Double.parseDouble(st.nextToken());
					st.nextToken(); 
					double cov = Double.parseDouble(st.nextToken());
					if (cov < thresh) continue;
					int abo = Integer.parseInt(st.nextToken());
					int est = Integer.parseInt(st.nextToken());

					if (chr == null) {
						chr = s;
						current = new BisCpG(pos, size);
						head = current;
						continue;
					}
					if (debug && pos == 3291740) System.out.println("Starting while");
					if (previous != null && previous.getPos() > pos) {
						System.err.println("Incorrect line order in " + input[i] + args[1] + "_meth_poor.txt (#" + lineCounts + "): " + line);
						current = head;
					}
					while (current != null && current.getPos() < pos) {
						if (debug && pos == 3291740 && current != null) System.out.println(current.getPos() + "\t" + pos);
						if (debug && pos == 3291740) System.out.print(current.getPos() + " ");
						previous = current;
						current = current.getNext();
					//	if (current != null && current.getPos() < previous.getPos()) {
					//		previous.setNext(null);
					//		current = null;
					//		break;
					//	}
					}
					if (debug && pos == 3291740) {
						System.out.println();
						System.out.println("Starting if");
						System.out.println("Previous: " + previous.getPos() + "; current: " + current.getPos() + "; next: " + current.getNext().getPos());
					}
					if (current == null || current.getPos() > pos) {
						BisCpG bc = new BisCpG(pos, size);
						total++;
						if (previous == null) {
							head = bc;
						} else {
							previous.setNext(bc);
						}
						bc.setNext(current);
						current = bc;
					}
					current.setCpG(i, beta, mval, abo, est);
					if (debug && pos == 3291740) {
						System.out.println("Done");
						System.out.println("Previous: " + previous.getPos() + "; current: " + current.getPos() + "; next: " + current.getNext().getPos());
					}
				}
				br.close();
				//System.out.println("Total lines read: " + lineCounts);
				//System.out.println("Total CpGs read: " + total);
				//current = head;
				//cur = 0;
				//while (current != null) {
				//	current = current.getNext();
				//	cur++;
				//}
				//System.out.println("Total CpGs in queue: " + cur);

			}
			
			PrintWriter pw = new PrintWriter(new FileWriter(output));
			pw.print("chrom\tpos");
			for (int i = 0; i < size; i++) 
			    pw.print("\t" + tag[i] + (count ? "_unmeth\t" : "_beta\t") + tag[i] + (count ? "_meth\t" : "_mval\t") + tag[i] + "_abolished\t" + tag[i] + "_established");
			pw.println();
			current = head;
			while (current != null) {
				pw.print(chr + "\t");
				if (count) {
				    pw.println(current.getCounts());
				} else {
				    pw.println(current);
				}
				if (current.getNext() != null && current.getNext().getPos() < current.getPos()) break;
				current = current.getNext();
			}
			pw.close();
		} catch (IOException ioe) {
			ioe.printStackTrace();
			System.exit(-1);
		} catch (Exception e) {
			System.err.println("Error in " + currentTag);
			System.err.println(line);
			e.printStackTrace();
			System.exit(-2);
		}
	}
}

class BisCpG {
	int pos;
	BisCpG next;
	double[] beta, mval;
	int[] abolished, established;
	
	BisCpG(int p, int size) {
		pos = p;
		beta = new double[size];
		for (int i = 0; i < size; i++) beta[i] = -1;
		mval = new double[size];
		abolished = new int[size];
		established = new int[size];
		next = null;
	}
	
	void setCpG(int idx, double b, double m, int a, int e) {
		beta[idx] = b;
		mval[idx] = m;
		abolished[idx] = a;
		established[idx] = e;
	}
	
	void setNext(BisCpG c) {
		next = c;
	}
	
	BisCpG getNext() {
		return next;
	}
	
	int getPos() {
		return pos;
	}
	
	public String toString() {
		String res = "" + pos;
		for (int i = 0; i < beta.length; i++) {
			if (beta[i] < 0) {
				res += "\tNA\tNA\tNA\tNA";
			} else {
				res += ("\t" + beta[i] + "\t" + mval[i] + "\t" + abolished[i] + "\t" + established[i]);
			}
		}
		return res;
	}
        public String getCounts() {
                String res = "" + pos;
                for (int i = 0; i < beta.length; i++) {
                        if (beta[i] < 0) {
                                res += "\tNA\tNA\tNA\tNA";
                        } else {
                                res += ("\t" + Math.round(beta[i]) + "\t" + Math.round(mval[i]) + "\t" + abolished[i] + "\t" + established[i]);
                        }
                }
                return res;
        }
}
