require(tree)||stop("Package tree not available. Please install the package")
require(weights)||stop("Package weights not available. Please install the package")
fun.help = function(pos, val, dev=1e-6, p.thresh=1e-9, mincut=5) {
	t = try(tree(val~pos, mindev=dev, mincut=mincut), silent=T)
	de = dev
	while (class(t) == "try-error"  & de < 1) {
	    de = de * 2
	    t = try(tree(val~pos, mindev=de, mincut=mincut), silent=T)
	}
	if (class(t) == "try-error") return (length(pos))
	t.p = try(prune.tree(t), silent=T)
	if (class(t.p) == "try-error") {
	    t.p.best = t
	} else {
	    t.p.bic = length(val)*log(t.p$dev/length(val)) + log(length(val)) * (t.p$size + 1)
	    while (which(t.p.bic==min(t.p.bic))[1] == 1 & de > dev/1000) {
		t1 = t
		t1.p = t.p
		de = de / 2
		t = try(tree(val~pos, mindev=de), silent=T)
		if (class(t) == "try-error") {
		    t = t1
		    t.p = t1.p
		    break
		}
		t.p = prune.tree(t)
		t.p.bic = length(val)*log(t.p$dev/length(val)) + log(length(val)) * (t.p$size + 1)
	    }
	
	    t.p.best = prune.tree(t, best= min(t.p$size[which(t.p.bic==min(t.p.bic, na.rm=T))]))
	}
	
	seg.chr = unique(t.p.best$where)
	for (zz in 1:length(seg.chr)) seg.chr[zz] = sum(t.p.best$where==seg.chr[zz])
	return(seg.chr)
}

fun.weighted = function(pos, val, weight, dev=1e-6, p.thresh=1e-9, mincut=5) {
	t = try(tree(val~pos, weights = weight, mindev=dev, mincut=mincut), silent=T)
	de = dev
	while (class(t) == "try-error"  & de < 1) {
	    de = de * 2
	    t = try(tree(val~pos, weights = weight, mindev=de, mincut=mincut), silent=T)
	}
	if (class(t) == "try-error") return (length(pos))
	t.p = try(prune.tree(t), silent=T)
	if (class(t.p) == "try-error") {
	    t.p.best = t
	} else {
	    t.p.bic = length(val)*log(t.p$dev/length(val)) + log(length(val)) * (t.p$size + 1)
	    while (which(t.p.bic==min(t.p.bic))[1] == 1 & de > dev/1000) {
		t1 = t
		t1.p = t.p
		de = de / 2
		t = try(tree(val~pos, weights = weight, mindev=de), silent=T)
		if (class(t) == "try-error") {
		    t = t1
		    t.p = t1.p
		    break
		}
		t.p = prune.tree(t)
		t.p.bic = length(val)*log(t.p$dev/length(val)) + log(length(val)) * (t.p$size + 1)
	    }
	
	    t.p.best = prune.tree(t, best= min(t.p$size[which(t.p.bic==min(t.p.bic, na.rm=T))]))
	}
	
	seg.chr = unique(t.p.best$where)
	for (zz in 1:length(seg.chr)) seg.chr[zz] = sum(t.p.best$where==seg.chr[zz])
	return(seg.chr)
}

fun.help2 = function(pos, val, dev=1e-6, beta.thresh=0.1, mincut=2) {
	t = try(tree(val~pos, mindev=dev, mincut=mincut), silent=T)
	de = dev
	while (class(t) == "try-error" & de < 1) {
	    de = de * 2
	    t = try(tree(val~pos, mindev=de, mincut=mincut), silent=T)
	}
	if (class(t) == "try-error") return (length(pos))
	seg.chr = unique(t$where)
	beta = seg.chr
	for (zz in 1:length(seg.chr)) {
		seg.idx = which(t$where==seg.chr[zz])
		seg.chr[zz] = length(seg.idx)
		beta[zz] = mean(val[seg.idx])
	}
	beta.diff = abs(diff(beta))
	while (length(beta) > 1 & min(beta.diff) < beta.thresh) {
		beta.idx = which(beta.diff == min(beta.diff))[1]
		beta[beta.idx] = (beta[beta.idx]*seg.chr[beta.idx] + beta[beta.idx+1]*seg.chr[beta.idx+1])/(seg.chr[beta.idx]+seg.chr[beta.idx+1])
		seg.chr[beta.idx] = seg.chr[beta.idx] + seg.chr[beta.idx+1]
		seg.chr = seg.chr[-(beta.idx+1)]
		beta = beta[-(beta.idx+1)]
		beta.diff = abs(diff(beta))
	}
	
	return(seg.chr)
}

fun.chr = function(chr, val, pos, diff, step=4000, overlap = round(step/20), p.thresh=1e-6, diff.thresh=0.1, debug=F, mincut=5) {
	na.idx = which(is.na(val))
	if (length(na.idx) > 0) {
		val2 = val[-na.idx]
		pos2 = pos[-na.idx]
		diff2 = diff[-na.idx]
		val2[which(val2 > 5)]=5
		val2[which(val2 < -5)]=-5
	} else {
		val2 = val
		pos2 = pos
		diff2 = diff
		val2[which(val2 > 5)]=5
		val2[which(val2 < -5)]=-5
	}
	current = 1
	offset = 0
	offset2 = 0
	num.mark1 = -1
	while (current <= length(val2)) {
	    if (current + step*1.5 - 1 < length(val2)) {
		val.tmp = val2[current + ((-offset):(step - 1))]
		pos.tmp = pos2[current + ((-offset):(step - 1))]
	 	current = current + step
	    } else {
		val.tmp = val2[(current - offset):length(val2)]
		pos.tmp = pos2[(current - offset):length(val2)]
		current = length(val2) + 1
	    }
	    mark.seg = fun.help(pos.tmp, val.tmp, p.thresh=p.thresh, mincut=mincut)
	    if (debug) print(paste("Finsihing", current, "bins.", date()))
	    if (num.mark1[1] < 0) {
		num.mark1 = mark.seg
	    } else {
		mark.seg[1] = mark.seg[1] + offset2
		if (length(num.mark1) > 1) {
		    num.mark1 = c(num.mark1[1:(length(num.mark1) - 1)], mark.seg)
		} else {
		    num.mark1 = mark.seg
		}
	    }
	    if (num.mark1[length(num.mark1)] > overlap) {
		offset = overlap
	    } else {
		offset = num.mark1[length(num.mark1)]
	    }
	    offset2 = num.mark1[length(num.mark1)] - offset
	}

	Start = array(1, dim=length(num.mark1))
	if (length(Start) > 1) {
	    for (i in 2 : length(num.mark1)) Start[i] = Start[i - 1] + num.mark1[i - 1]
		for (i in 1 : (length(num.mark1) - 1)) {
			if (abs(mean(val2[Start[i]:(Start[i] + num.mark1[i] - 2)]) - val2[Start[i+1] - 1]) > abs(mean(val2[Start[i+1]:(Start[i+1] + num.mark1[i+1] - 1)]) - val2[Start[i+1] - 1])) {
				num.mark1[i] = num.mark1[i] - 1
				num.mark1[i+1] = num.mark1[i+1] + 1
				Start[i+1] = Start[i+1] - 1
			} else if (abs(mean(val2[Start[i]:(Start[i] + num.mark1[i] - 1)]) - val2[Start[i+1]]) < abs(mean(val2[(Start[i+1]+1):(Start[i+1] + num.mark1[i+1] - 1)]) - val2[Start[i+1]])) {
				num.mark1[i] = num.mark1[i] + 1
				num.mark1[i+1] = num.mark1[i+1] - 1
				Start[i+1] = Start[i+1] + 1
			}
		}
		idx=which(num.mark1==0)
		if (length(idx) > 0) {
			Start = Start[-idx]
			num.mark1 = num.mark1[-idx]
		}
	    p.val = array(1, length(num.mark1)-1)
	    for (i in 1: length(p.val)) {
			multiplier = 1
			if (num.mark1[i] == 1 || num.mark1[i+1] == 1) multiplier = 5
			if ((num.mark1[i] == 1 || sd(val2[Start[i]:(Start[i] + num.mark1[i] - 1)]) == 0) && (num.mark1[i+1] == 1 || sd(val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)]) == 0)) {
				p.val[i] = p.thresh/1000
				if (abs(val2[Start[i]]-val2[Start[i + 1]]) < multiplier * diff.thresh) p.val[i] = 1 - abs(val2[Start[i]]-val2[Start[i + 1]])
			} else {
				if (num.mark1[i] == 1) {
					p.val[i] = t.test(mu=val2[Start[i]], val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)])$p.value * 100
				} else if (num.mark1[i + 1] == 1) {
					p.val[i] = t.test(val2[Start[i]:(Start[i] + num.mark1[i] - 1)], mu=val2[Start[i + 1]])$p.value * 100
				} else {
					p.val[i] = t.test(val2[Start[i]:(Start[i] + num.mark1[i] - 1)], val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)])$p.value
				}
				if (abs(mean(diff2[Start[i]:(Start[i] + num.mark1[i] - 1)]) - mean(diff2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)])) < multiplier * diff.thresh) p.val[i] = p.val[i] + 2 * p.thresh
			}
		}
	    p.max.id = which(p.val == max(p.val))[1]
	    while (max(p.val) > p.thresh) {
	    	num.mark1[p.max.id] = num.mark1[p.max.id] + num.mark1[p.max.id+1]
			num.mark1 = num.mark1[-(p.max.id + 1)]
	    	p.val = p.val[-p.max.id]
	    	Start = Start[-(p.max.id+1)]
	    	if (p.max.id > 1) {
				multiplier = 1
				if (num.mark1[p.max.id - 1] == 1) multiplier = 5
				if ((num.mark1[p.max.id - 1] == 1 || sd(val2[Start[p.max.id-1]:(Start[p.max.id - 1] + num.mark1[p.max.id - 1] - 1)])) == 0 && sd(val2[Start[p.max.id]:(Start[p.max.id] + num.mark1[p.max.id] - 1)]) == 0) {
					p.val[p.max.id-1] = p.thresh/1000
					if (val2[Start[p.max.id-1]] == val2[Start[p.max.id]]) p.val[p.max.id-1] = 1
				} else {
					if (num.mark1[p.max.id - 1] == 1) {
						p.val[p.max.id-1] = t.test(mu=val2[Start[p.max.id-1]], val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])$p.value * 100
					} else {
						p.val[p.max.id-1] = t.test(val2[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)], val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])$p.value
					}
					if (abs(mean(diff2[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)]) - mean(diff2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])) < multiplier * diff.thresh) p.val[p.max.id-1] = p.val[p.max.id-1] + 2 * p.thresh
				}
			}
	    	if (p.max.id <= length(p.val)) {
				multiplier = 1
				if (num.mark1[p.max.id + 1] == 1) multiplier = 5
				if (sd(val2[Start[p.max.id]:(Start[p.max.id] + num.mark1[p.max.id] - 1)]) == 0 && (num.mark1[p.max.id+1] == 1 || sd(val2[Start[p.max.id+1]:(Start[p.max.id+1] + num.mark1[p.max.id+1] - 1)]) == 0)) {
					p.val[p.max.id] = p.thresh/1000
					if (val2[Start[p.max.id]] == val2[Start[p.max.id+1]]) p.val[p.max.id] = 1
				} else {
					if (num.mark1[p.max.id+1] == 1) {
						p.val[p.max.id] = t.test(val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], mu=val2[Start[p.max.id+1]])$p.value * 100
					} else {
						p.val[p.max.id] = t.test(val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], val2[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)])$p.value
					}
					if (abs(mean(diff2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)]) - mean(diff2[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)])) < multiplier * diff.thresh) p.val[p.max.id] = p.val[p.max.id] + 2 * p.thresh
				}
			}
	    	p.max.id = which(p.val == max(p.val))[1]
	    }
	}
	
	seg.chr =  data.frame(matrix(0, nrow=length(num.mark1), ncol=6, dimnames = list(NULL, c("chrom", "loc.start", "loc.end", "num.mark", "beta.logit.mean", "beta.mean"))))
	seg.chr$num.mark = num.mark1
	seg.chr$chrom = chr
	Start = array(1, dim=nrow(seg.chr))
	if (nrow(seg.chr) > 1) {
	    for (i in 2 : nrow(seg.chr)) Start[i] = Start[i - 1] + seg.chr$num.mark[i - 1]
	}
	seg.chr$loc.start = pos2[Start]
	seg.chr$loc.end = pos2[Start + seg.chr$num.mark - 1]
	for (j in 1:length(Start)) {
	    seg.chr$beta.logit.mean[j] = mean(val2[Start[j]:(Start[j] + seg.chr$num.mark[j] - 1)], na.rm=T)
		seg.chr$beta.mean[j] = mean(diff2[Start[j]:(Start[j] + seg.chr$num.mark[j] - 1)], na.rm=T)
	}
	return(seg.chr)
}

fun_SD.chr = function(chr, val, pos, diff, step=4000, overlap = round(step/20), p.thresh=1e-6, diff.thresh=0.1, debug=F, mincut=5, SD.length = 10000) {
	na.idx = which(is.na(val))
	if (length(na.idx) > 0) {
		val2 = val[-na.idx]
		pos2 = pos[-na.idx]
		diff2 = diff[-na.idx]
		val2[which(val2 > 5)]=5
		val2[which(val2 < -5)]=-5
	} else {
		val2 = val
		pos2 = pos
		diff2 = diff
		val2[which(val2 > 5)]=5
		val2[which(val2 < -5)]=-5
	}
	current = 1
	offset = 0
	offset2 = 0
	num.mark1 = -1
	while (current <= length(val2)) {
	    if (current + step*1.5 - 1 < length(val2)) {
		val.tmp = val2[current + ((-offset):(step - 1))]
		pos.tmp = pos2[current + ((-offset):(step - 1))]
	 	current = current + step
	    } else {
		val.tmp = val2[(current - offset):length(val2)]
		pos.tmp = pos2[(current - offset):length(val2)]
		current = length(val2) + 1
	    }
	    mark.seg = fun.help(pos.tmp, val.tmp, p.thresh=p.thresh, mincut=mincut)
	    if (debug) print(paste("Finsihing", current, "bins.", date()))
	    if (num.mark1[1] < 0) {
		num.mark1 = mark.seg
	    } else {
		mark.seg[1] = mark.seg[1] + offset2
		if (length(num.mark1) > 1) {
		    num.mark1 = c(num.mark1[1:(length(num.mark1) - 1)], mark.seg)
		} else {
		    num.mark1 = mark.seg
		}
	    }
	    if (num.mark1[length(num.mark1)] > overlap) {
		offset = overlap
	    } else {
		offset = num.mark1[length(num.mark1)]
	    }
	    offset2 = num.mark1[length(num.mark1)] - offset
	}

	Start = array(1, dim=length(num.mark1))
	if (length(Start) > 1) {
	    for (i in 2 : length(num.mark1)) Start[i] = Start[i - 1] + num.mark1[i - 1]
		for (i in 1 : (length(num.mark1) - 1)) {
			if (abs(mean(val2[Start[i]:(Start[i] + num.mark1[i] - 2)]) - val2[Start[i+1] - 1]) > abs(mean(val2[Start[i+1]:(Start[i+1] + num.mark1[i+1] - 1)]) - val2[Start[i+1] - 1])) {
				num.mark1[i] = num.mark1[i] - 1
				num.mark1[i+1] = num.mark1[i+1] + 1
				Start[i+1] = Start[i+1] - 1
			} else if (abs(mean(val2[Start[i]:(Start[i] + num.mark1[i] - 1)]) - val2[Start[i+1]]) < abs(mean(val2[(Start[i+1]+1):(Start[i+1] + num.mark1[i+1] - 1)]) - val2[Start[i+1]])) {
				num.mark1[i] = num.mark1[i] + 1
				num.mark1[i+1] = num.mark1[i+1] - 1
				Start[i+1] = Start[i+1] + 1
			}
		}
		idx=which(num.mark1==0)
		if (length(idx) > 0) {
			Start = Start[-idx]
			num.mark1 = num.mark1[-idx]
		}
	    p.val = array(1, length(num.mark1)-1)
	    for (i in 1: length(p.val)) {
			multiplier = 1
			if (num.mark1[i] == 1 || num.mark1[i+1] == 1) multiplier = 5
			if ((num.mark1[i] == 1 || sd(val2[Start[i]:(Start[i] + num.mark1[i] - 1)]) == 0) && (num.mark1[i+1] == 1 || sd(val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)]) == 0)) {
				p.val[i] = p.thresh/1000
				if (abs(val2[Start[i]]-val2[Start[i + 1]]) < multiplier * diff.thresh) p.val[i] = 1 - abs(val2[Start[i]]-val2[Start[i + 1]])
			} else {
				if (num.mark1[i] == 1) {
					p.val[i] = t.test(mu=val2[Start[i]], val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)])$p.value * 100
				} else if (num.mark1[i + 1] == 1) {
					p.val[i] = t.test(val2[Start[i]:(Start[i] + num.mark1[i] - 1)], mu=val2[Start[i + 1]])$p.value * 100
				} else {
					p.val[i] = t.test(val2[Start[i]:(Start[i] + num.mark1[i] - 1)], val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)])$p.value
				}
				if (abs(mean(diff2[Start[i]:(Start[i] + num.mark1[i] - 1)]) - mean(diff2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)])) < multiplier * diff.thresh) p.val[i] = p.val[i] + 2 * p.thresh
			}
		}
	    p.max.id = which(p.val == max(p.val))[1]
	    while (max(p.val) > p.thresh) {
	    	num.mark1[p.max.id] = num.mark1[p.max.id] + num.mark1[p.max.id+1]
			num.mark1 = num.mark1[-(p.max.id + 1)]
	    	p.val = p.val[-p.max.id]
	    	Start = Start[-(p.max.id+1)]
	    	if (p.max.id > 1) {
				multiplier = 1
				if (num.mark1[p.max.id - 1] == 1) multiplier = 5
				if ((num.mark1[p.max.id - 1] == 1 || sd(val2[Start[p.max.id-1]:(Start[p.max.id - 1] + num.mark1[p.max.id - 1] - 1)])) == 0 && sd(val2[Start[p.max.id]:(Start[p.max.id] + num.mark1[p.max.id] - 1)]) == 0) {
					p.val[p.max.id-1] = p.thresh/1000
					if (val2[Start[p.max.id-1]] == val2[Start[p.max.id]]) p.val[p.max.id-1] = 1
				} else {
					if (num.mark1[p.max.id - 1] == 1) {
						p.val[p.max.id-1] = t.test(mu=val2[Start[p.max.id-1]], val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])$p.value * 100
					} else {
						p.val[p.max.id-1] = t.test(val2[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)], val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])$p.value
					}
					if (abs(mean(diff2[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)]) - mean(diff2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])) < multiplier * diff.thresh) p.val[p.max.id-1] = p.val[p.max.id-1] + 2 * p.thresh
				}
			}
	    	if (p.max.id <= length(p.val)) {
				multiplier = 1
				if (num.mark1[p.max.id + 1] == 1) multiplier = 5
				if (sd(val2[Start[p.max.id]:(Start[p.max.id] + num.mark1[p.max.id] - 1)]) == 0 && (num.mark1[p.max.id+1] == 1 || sd(val2[Start[p.max.id+1]:(Start[p.max.id+1] + num.mark1[p.max.id+1] - 1)]) == 0)) {
					p.val[p.max.id] = p.thresh/1000
					if (val2[Start[p.max.id]] == val2[Start[p.max.id+1]]) p.val[p.max.id] = 1
				} else {
					if (num.mark1[p.max.id+1] == 1) {
						p.val[p.max.id] = t.test(val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], mu=val2[Start[p.max.id+1]])$p.value * 100
					} else {
						p.val[p.max.id] = t.test(val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], val2[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)])$p.value
					}
					if (abs(mean(diff2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)]) - mean(diff2[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)])) < multiplier * diff.thresh) p.val[p.max.id] = p.val[p.max.id] + 2 * p.thresh
				}
			}
	    	p.max.id = which(p.val == max(p.val))[1]
	    }
	}
	
	seg.chr =  data.frame(matrix(0, nrow=length(num.mark1), ncol=8, dimnames = list(NULL, c("chrom", "loc.start", "loc.end", "num.mark", "beta.logit.mean", "beta.mean", "SD_l", "SD_r"))))
	seg.chr$num.mark = num.mark1
	seg.chr$chrom = chr
	Start = array(1, dim=nrow(seg.chr))
	if (nrow(seg.chr) > 1) {
	    for (i in 2 : nrow(seg.chr)) Start[i] = Start[i - 1] + seg.chr$num.mark[i - 1]
	}
	seg.chr$loc.start = pos2[Start]
	seg.chr$loc.end = pos2[Start + seg.chr$num.mark - 1]
	for (j in 1:length(Start)) {
	    seg.chr$beta.logit.mean[j] = mean(val2[Start[j]:(Start[j] + seg.chr$num.mark[j] - 1)], na.rm=T)
		seg.chr$beta.mean[j] = mean(diff2[Start[j]:(Start[j] + seg.chr$num.mark[j] - 1)], na.rm=T)
	}
	for (i in 1 : nrow(seg.chr)) {
	    idx = which(pos2 > seg.chr$loc.start[i] - SD.length & pos2 < seg.chr$loc.start[i])
	    if (length(idx) < 20) {
		seg.chr$SD_l[i] = NA
	    } else {
		seg.chr$SD_l[i] = sd(diff2[idx])
	    }
	    idx = which(pos2 > seg.chr$loc.end[i] & pos2 < seg.chr$loc.end[i] + SD.length)
	    if (length(idx) < 20) {
		seg.chr$SD_r[i] = NA
	    } else {
		seg.chr$SD_r[i] = sd(diff2[idx])
	    }
	    
	}
	return(seg.chr)
}

tree.weighted = function(chr, val, val.weight, pos, diff, diff.weight, step=4000, overlap = round(step/20), p.thresh=1e-6, diff.thresh=0.1, debug=F, mincut=5, option=0) {
	na.idx = which(is.na(val))
	if (length(na.idx) > 0) {
		val2 = val[-na.idx]
		val2.weight = val.weight[-na.idx]
		pos2 = pos[-na.idx]
		diff2 = diff[-na.idx]
		diff2.weight = diff.weight[-na.idx]
		val2[which(val2 > 5)]=5
		val2[which(val2 < -5)]=-5
	} else {
		val2 = val
		pos2 = pos
		diff2 = diff
		val2.weight = val.weight
		diff2.weight = diff.weight
		val2[which(val2 > 5)]=5
		val2[which(val2 < -5)]=-5
	}
	current = 1
	offset = 0
	offset2 = 0
	num.mark1 = -1
	while (current <= length(val2)) {
	    if (current + step*1.5 - 1 < length(val2)) {
		val.tmp = val2[current + ((-offset):(step - 1))]
		val.tmp.weight = val2.weight[current + ((-offset):(step - 1))]
		pos.tmp = pos2[current + ((-offset):(step - 1))]
	 	current = current + step
	    } else {
		val.tmp = val2[(current - offset):length(val2)]
		val.tmp.weight = val2.weight[(current - offset):length(val2)]
		pos.tmp = pos2[(current - offset):length(val2)]
		current = length(val2) + 1
	    }
		if (option > 0) {
			mark.seg = fun.help(pos.tmp, val.tmp, p.thresh=p.thresh, mincut=mincut)
		} else {
			mark.seg = fun.weighted(pos.tmp, val.tmp, val.tmp.weight, p.thresh=p.thresh, mincut=mincut)
		}
	    if (debug) print(paste("Finsihing", current, "bins.", date()))
	    if (num.mark1[1] < 0) {
		num.mark1 = mark.seg
	    } else {
		mark.seg[1] = mark.seg[1] + offset2
		if (length(num.mark1) > 1) {
		    num.mark1 = c(num.mark1[1:(length(num.mark1) - 1)], mark.seg)
		} else {
		    num.mark1 = mark.seg
		}
	    }
	    if (num.mark1[length(num.mark1)] > overlap) {
		offset = overlap
	    } else {
		offset = num.mark1[length(num.mark1)]
	    }
	    offset2 = num.mark1[length(num.mark1)] - offset
	}

	Start = array(1, dim=length(num.mark1))
	if (length(Start) > 1) {
	    for (i in 2 : length(num.mark1)) Start[i] = Start[i - 1] + num.mark1[i - 1]
		for (i in 1 : (length(num.mark1) - 1)) {
			if (abs(mean(val2[Start[i]:(Start[i] + num.mark1[i] - 2)]) - val2[Start[i+1] - 1]) > abs(mean(val2[Start[i+1]:(Start[i+1] + num.mark1[i+1] - 1)]) - val2[Start[i+1] - 1])) {
				num.mark1[i] = num.mark1[i] - 1
				num.mark1[i+1] = num.mark1[i+1] + 1
				Start[i+1] = Start[i+1] - 1
			} else if (abs(mean(val2[Start[i]:(Start[i] + num.mark1[i] - 1)]) - val2[Start[i+1]]) < abs(mean(val2[(Start[i+1]+1):(Start[i+1] + num.mark1[i+1] - 1)]) - val2[Start[i+1]])) {
				num.mark1[i] = num.mark1[i] + 1
				num.mark1[i+1] = num.mark1[i+1] - 1
				Start[i+1] = Start[i+1] + 1
			}
		}
		idx=which(num.mark1==0)
		if (length(idx) > 0) {
			Start = Start[-idx]
			num.mark1 = num.mark1[-idx]
		}
	    p.val = array(1, length(num.mark1)-1)
	    for (i in 1: length(p.val)) {
			multiplier = 1
			if (num.mark1[i] == 1 || num.mark1[i+1] == 1) multiplier = 5
			if ((num.mark1[i] == 1 || sd(val2[Start[i]:(Start[i] + num.mark1[i] - 1)]) == 0) && (num.mark1[i+1] == 1 || sd(val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)]) == 0)) {
				p.val[i] = p.thresh/1000
				if (abs(val2[Start[i]]-val2[Start[i + 1]]) < multiplier * diff.thresh) p.val[i] = 1 - abs(val2[Start[i]]-val2[Start[i + 1]])
			} else {
				if (num.mark1[i] == 1) {
					test.model = wtd.t.test(val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)], val2[Start[i]], weighty=val2.weight[Start[i]], weight=val2.weight[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)])
				} else {
					test.model = wtd.t.test(val2[Start[i]:(Start[i] + num.mark1[i] - 1)], val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)], weight=val2.weight[Start[i]:(Start[i] + num.mark1[i] - 1)], weighty=val2.weight[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)])
				}
				if (is.na(test.model$coefficients[3])) {
					weight.min = min(val2.weight[Start[i]:(Start[i + 1] + num.mark1[i + 1] - 1)])
					if (num.mark1[i] == 1) {
						test.model = wtd.t.test(val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)], val2[Start[i]], weighty=val2.weight[Start[i]]/weight.min, weight=val2.weight[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)]/weight.min)
					} else {
						test.model = wtd.t.test(val2[Start[i]:(Start[i] + num.mark1[i] - 1)], val2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)], weight=val2.weight[Start[i]:(Start[i] + num.mark1[i] - 1)]/weight.min, weighty=val2.weight[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)]/weight.min)
					}
				}
				p.val[i] = test.model$coefficients[3]
				if (num.mark1[i] == 1 || num.mark1[i+1] == 1) p.val[i] = p.val[i] * 100
				if (abs(weighted.mean(diff2[Start[i]:(Start[i] + num.mark1[i] - 1)], w=diff2.weight[Start[i]:(Start[i] + num.mark1[i] - 1)]) - weighted.mean(diff2[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)], w=diff2.weight[Start[i + 1]:(Start[i + 1] + num.mark1[i + 1] - 1)])) < multiplier * diff.thresh) p.val[i] = p.val[i] + 2 * p.thresh
			}
		}
	    p.max.id = which(p.val == max(p.val))[1]
	    while (max(p.val) > p.thresh) {
	    	num.mark1[p.max.id] = num.mark1[p.max.id] + num.mark1[p.max.id+1]
			num.mark1 = num.mark1[-(p.max.id + 1)]
	    	p.val = p.val[-p.max.id]
	    	Start = Start[-(p.max.id+1)]
	    	if (p.max.id > 1) {
				multiplier = 1
				if (num.mark1[p.max.id - 1] == 1) multiplier = 5
				if ((num.mark1[p.max.id - 1] == 1 || sd(val2[Start[p.max.id-1]:(Start[p.max.id - 1] + num.mark1[p.max.id - 1] - 1)]) == 0) & sd(val2[Start[p.max.id]:(Start[p.max.id] + num.mark1[p.max.id] - 1)]) == 0) {
					p.val[p.max.id-1] = p.thresh/1000
					if (val2[Start[p.max.id-1]] == val2[Start[p.max.id]]) p.val[p.max.id-1] = 1
				} else {
					if (num.mark1[p.max.id - 1] == 1) {
						test.model=wtd.t.test(val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], val2[Start[p.max.id-1]], weighty=val2.weight[Start[p.max.id-1]], weight=val2.weight[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])
					} else {
						test.model=wtd.t.test(val2[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)], val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], weight=val2.weight[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)], weighty=val2.weight[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])
					}
					if (is.na(test.model$coefficients[3])) {
						weight.min = min(val2.weight[Start[p.max.id-1]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])
						if (num.mark1[p.max.id - 1] == 1) {
							test.model=wtd.t.test(val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], val2[Start[p.max.id-1]], weighty=val2.weight[Start[p.max.id-1]]/weight.min, weight=val2.weight[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)]/weight.min)
						} else {
							test.model=wtd.t.test(val2[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)], val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], weight=val2.weight[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)]/weight.min, weighty=val2.weight[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)]/weight.min)
						}
					}
					p.val[p.max.id-1] = test.model$coefficients[3]
					if (num.mark1[p.max.id - 1] == 1) p.val[p.max.id-1] = p.val[p.max.id-1] * 100
					if (abs(weighted.mean(diff2[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)], w=diff2.weight[Start[p.max.id-1]:(num.mark1[p.max.id - 1] + Start[p.max.id-1] - 1)])-weighted.mean(diff2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], w=diff2.weight[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])) < multiplier * diff.thresh) p.val[p.max.id-1] = p.val[p.max.id-1] + 2 * p.thresh
				}
			}
	    	if (p.max.id <= length(p.val)) {
				multiplier = 1
				if (num.mark1[p.max.id + 1] == 1) multiplier = 5
				if (sd(val2[Start[p.max.id]:(Start[p.max.id] + num.mark1[p.max.id] - 1)]) == 0 && (num.mark1[p.max.id+1] == 1 || sd(val2[Start[p.max.id+1]:(Start[p.max.id+1] + num.mark1[p.max.id+1] - 1)]) == 0)) {
					p.val[p.max.id] = p.thresh/1000
					if (val2[Start[p.max.id]] == val2[Start[p.max.id+1]]) p.val[p.max.id] = 1
				} else {
					test.model = wtd.t.test(val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], val2[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)], weight=val2.weight[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], weighty=val2.weight[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)])
					if (is.na(test.model$coefficients[3])) {
						weight.min = min(val2.weight[Start[p.max.id]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)])
						test.model = wtd.t.test(val2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], val2[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)], weight=val2.weight[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)]/weight.min, weighty=val2.weight[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)]/weight.min)
					}
					p.val[p.max.id] = test.model$coefficients[3]
					if (num.mark1[p.max.id + 1] == 1) p.val[p.max.id] = p.val[p.max.id] * 100
					if (abs(weighted.mean(diff2[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)], w=diff2.weight[Start[p.max.id]:(num.mark1[p.max.id] + Start[p.max.id] - 1)])-weighted.mean(diff2[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)], w=diff2.weight[Start[p.max.id+1]:(num.mark1[p.max.id+1] + Start[p.max.id+1] - 1)])) < multiplier * diff.thresh) p.val[p.max.id] = p.val[p.max.id] + 2 * p.thresh
				}
			}
	    	p.max.id = which(p.val == max(p.val))[1]
	    }
	}
	
	seg.chr =  data.frame(matrix(0, nrow=length(num.mark1), ncol=6, dimnames = list(NULL, c("chrom", "loc.start", "loc.end", "num.mark", "beta.logit.mean", "beta.mean"))))
	seg.chr$num.mark = num.mark1
	seg.chr$chrom = chr
	Start = array(1, dim=nrow(seg.chr))
	if (nrow(seg.chr) > 1) {
	    for (i in 2 : nrow(seg.chr)) Start[i] = Start[i - 1] + seg.chr$num.mark[i - 1]
	}
	seg.chr$loc.start = pos2[Start]
	seg.chr$loc.end = pos2[Start + seg.chr$num.mark - 1]
	for (j in 1:length(Start)) {
	    seg.chr$beta.logit.mean[j] = weighted.mean(val2[Start[j]:(Start[j] + seg.chr$num.mark[j] - 1)], w=val2.weight[Start[j]:(Start[j] + seg.chr$num.mark[j] - 1)], na.rm=T)
		seg.chr$beta.mean[j] = weighted.mean(diff2[Start[j]:(Start[j] + seg.chr$num.mark[j] - 1)], w=diff2.weight[Start[j]:(Start[j] + seg.chr$num.mark[j] - 1)], na.rm=T)
	}
	return(seg.chr)
}

fun.tmp = function(data, int1, int2) {
	return (max(0, as.double(data[int2])-as.double(data[int1])))
}


sd.fun = function(x) {
	if (sum(is.na(x)) >= length(x) / 2) {
		return(NA) 
	} else {
		return(sd(x, na.rm=T))
	}
}
mean.fun = function(x) {
	if (sum(is.na(x)) >= length(x) / 2) {
		return(NA) 
	} else {
		return(mean(x, na.rm=T))
	}
}

plotRegion = function(filename, loc.start, loc.end) {
	png(filename, width=2000, height=2500)
	par(mfrow=c(4,1))
	par(cex=2)
	plot(chr21$pos, chr21[,3], "p", pch=20, col="gray", xlim=c(loc.start, loc.end), xlab="Position", ylab="beta", main = "Set4, Day0, chr21")
	plot(chr21$pos, chr21[,5], "p", pch=20, col="gray", xlim=c(loc.start, loc.end), xlab="Position", ylab="beta", main = "Set5, Day0, chr21")
	plot(chr21$pos, chr21[,7], "p", pch=20, col="gray", xlim=c(loc.start, loc.end), xlab="Position", ylab="beta", main = "Set6, Day0, chr21")
	plot(chr21$pos, Day0.bval, "p", pch=20, col="gray", xlim=c(loc.start, loc.end), xlab="Position", ylab="beta", main = "Merged, Day0, chr21")
	idx = which(seg_noweight.chr21$loc.start >= loc.start & seg_noweight.chr21$loc.end <= loc.end)
	for (i in idx) {
		lines(seg_noweight.chr21[i, 2:3], rep(seg_noweight.chr21$beta.mean[i], 2), lwd=2, col="blue")
	}
	idx = which(seg_partial.chr21$loc.start >= loc.start & seg_partial.chr21$loc.end <= loc.end)
	for (i in idx) {
		lines(seg_partial.chr21[i, 2:3], rep(seg_partial.chr21$beta.mean[i], 2), lwd=2, col="red")
	}
	dev.off()
}

my.t.test.p.value <- function(...) {
   obj<-try(t.test(...), silent=TRUE)
   if (is(obj, "try-error")) return(NA) else return(obj$p.value)
}

ttest.fun = function(x, idx1, idx2) {
    if (sum(is.na(x[idx1])) + sum(is.na(x[idx2])) > 1) return(-1)
    if (abs(mean(as.double(x[idx1-1])-as.double(x[idx2-1]), na.rm=T)) < 0.1) return(1)
    return(my.t.test.p.value(x[idx1], x[idx2], paired=T))
}
triplicate.process = function(chr, filename, ord) {
    chr.data = read.table(filename, header=T)
    chr.data = chr.data[, ord]
    Day12.mval.sd = apply(chr.data[,c(16,18,20)], 1, sd.fun)
    Day12.mval.sd = Day12.mval.sd + summary(Day12.mval.sd)[2]
    Day12.mval.weight = 1/Day12.mval.sd
    Day12.mval.weight = Day12.mval.weight / exp(mean(log(Day12.mval.weight), na.rm=T))
    Day12.bval.sd = apply(chr.data[,c(15,17,19)], 1, sd.fun)
    Day12.bval.sd = Day12.bval.sd + max(0.01, summary(Day12.bval.sd)[2])
    Day12.bval.weight = 1/Day12.bval.sd
    Day12.bval.weight = Day12.bval.weight / exp(mean(log(Day12.bval.weight), na.rm=T))

    Day12.mval = apply(chr.data[,c(16,18,20)], 1, mean.fun)
    Day12.bval = apply(chr.data[,c(15,17,19)], 1, mean.fun)
    seg_partial.chr.data.day12=tree.weighted(chr, Day12.mval, Day12.mval.weight, chr.data$pos, Day12.bval, Day12.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.day12, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Day12_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Day3.mval.sd = apply(chr.data[,c(10,12,14)], 1, sd.fun)
    Day3.mval.sd = Day3.mval.sd + summary(Day3.mval.sd)[2]
    Day3.mval.weight = 1/Day3.mval.sd
    Day3.mval.weight = Day3.mval.weight / exp(mean(log(Day3.mval.weight), na.rm=T))
    Day3.bval.sd = apply(chr.data[,c(9,11,13)], 1, sd.fun)
    Day3.bval.sd = Day3.bval.sd + max(0.01, summary(Day3.bval.sd)[2])
    Day3.bval.weight = 1/Day3.bval.sd
    Day3.bval.weight = Day3.bval.weight / exp(mean(log(Day3.bval.weight), na.rm=T))

    Day3.mval = apply(chr.data[,c(10,12,14)], 1, mean.fun)
    Day3.bval = apply(chr.data[,c(9,11,13)], 1, mean.fun)
    seg_partial.chr.data.day3=tree.weighted(chr, Day3.mval, Day3.mval.weight, chr.data$pos, Day3.bval, Day3.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.day3, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Day3_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Day0.mval.sd = apply(chr.data[,c(4,6,8)], 1, sd.fun)
    Day0.mval.sd = Day0.mval.sd + summary(Day0.mval.sd)[2]
    Day0.mval.weight = 1/Day0.mval.sd
    Day0.mval.weight = Day0.mval.weight / exp(mean(log(Day0.mval.weight), na.rm=T))
    Day0.bval.sd = apply(chr.data[,c(3,5,7)], 1, sd.fun)
    Day0.bval.sd = Day0.bval.sd + max(0.01, summary(Day0.bval.sd)[2])
    Day0.bval.weight = 1/Day0.bval.sd
    Day0.bval.weight = Day0.bval.weight / exp(mean(log(Day0.bval.weight), na.rm=T))

    Day0.mval = apply(chr.data[,c(4,6,8)], 1, mean.fun)
    Day0.bval = apply(chr.data[,c(3,5,7)], 1, mean.fun)
    seg_partial.chr.data.day0=tree.weighted(chr, Day0.mval, Day0.mval.weight, chr.data$pos, Day0.bval, Day0.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.day0, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Day0_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Set5.mval.sd = apply(chr.data[,c(6,12,18)], 1, sd.fun)
    Set5.mval.sd = Set5.mval.sd + summary(Set5.mval.sd)[2]
    Set5.mval.weight = 1/Set5.mval.sd
    Set5.mval.weight = Set5.mval.weight / exp(mean(log(Set5.mval.weight), na.rm=T))
    Set5.bval.sd = apply(chr.data[,c(5,11,17)], 1, sd.fun)
    Set5.bval.sd = Set5.bval.sd + max(0.01, summary(Set5.bval.sd)[2])
    Set5.bval.weight = 1/Set5.bval.sd
    Set5.bval.weight = Set5.bval.weight / exp(mean(log(Set5.bval.weight), na.rm=T))

    Set5.mval = apply(chr.data[,c(6,12,18)], 1, mean.fun)
    Set5.bval = apply(chr.data[,c(5,11,17)], 1, mean.fun)
    seg_partial.chr.data.set5=tree.weighted(chr, Set5.mval, Set5.mval.weight, chr.data$pos, Set5.bval, Set5.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.set5, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set5_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Set4.mval.sd = apply(chr.data[,c(4,10,16)], 1, sd.fun)
    Set4.mval.sd = Set4.mval.sd + summary(Set4.mval.sd)[2]
    Set4.mval.weight = 1/Set4.mval.sd
    Set4.mval.weight = Set4.mval.weight / exp(mean(log(Set4.mval.weight), na.rm=T))
    Set4.bval.sd = apply(chr.data[,c(3,9,15)], 1, sd.fun)
    Set4.bval.sd = Set4.bval.sd + max(0.01, summary(Set4.bval.sd)[2])
    Set4.bval.weight = 1/Set4.bval.sd
    Set4.bval.weight = Set4.bval.weight / exp(mean(log(Set4.bval.weight), na.rm=T))

    Set4.mval = apply(chr.data[,c(4,10,16)], 1, mean.fun)
    Set4.bval = apply(chr.data[,c(3,9,15)], 1, mean.fun)
    seg_partial.chr.data.Set4=tree.weighted(chr, Set4.mval, Set4.mval.weight, chr.data$pos, Set4.bval, Set4.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.Set4, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set4_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Set6.mval.sd = apply(chr.data[,c(8,14,20)], 1, sd.fun)
    Set6.mval.sd = Set6.mval.sd + summary(Set6.mval.sd)[2]
    Set6.mval.weight = 1/Set6.mval.sd
    Set6.mval.weight = Set6.mval.weight / exp(mean(log(Set6.mval.weight), na.rm=T))
    Set6.bval.sd = apply(chr.data[,c(7,13,19)], 1, sd.fun)
    Set6.bval.sd = Set6.bval.sd + max(0.01, summary(Set6.bval.sd)[2])
    Set6.bval.weight = 1/Set6.bval.sd
    Set6.bval.weight = Set6.bval.weight / exp(mean(log(Set6.bval.weight), na.rm=T))

    Set6.mval = apply(chr.data[,c(8,14,20)], 1, mean.fun)
    Set6.bval = apply(chr.data[,c(7,13,19)], 1, mean.fun)
    seg_partial.chr.data.Set6=tree.weighted(chr, Set6.mval, Set6.mval.weight, chr.data$pos, Set6.bval, Set6.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.Set6, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set6_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")
    gc()
}

triplicate.diff = function(chr, filename, ord) {
    chr.data = read.table(filename, header=T)
    chr.data = chr.data[, ord]
    Day12diff.mval.sd = apply(chr.data[,c(16,18,20)] - chr.data[,c(4,6,8)], 1, sd.fun)
    Day12diff.mval.sd = Day12diff.mval.sd + summary(Day12diff.mval.sd)[2]
    Day12diff.mval.weight = 1/Day12diff.mval.sd
    Day12diff.mval.weight = Day12diff.mval.weight / exp(mean(log(Day12diff.mval.weight), na.rm=T))
    Day12diff.bval.sd = apply(chr.data[,c(15,17,19)] - chr.data[,c(3,5,7)], 1, sd.fun)
    Day12diff.bval.sd = Day12diff.bval.sd + summary(Day12diff.bval.sd)[2]
    Day12diff.bval.weight = 1/Day12diff.bval.sd
    Day12diff.bval.weight = Day12diff.bval.weight / exp(mean(log(Day12diff.bval.weight), na.rm=T))

    Day12diff.mval = apply(chr.data[,c(16,18,20)]-chr.data[,c(4,6,8)], 1, mean.fun)
    Day12diff.bval = apply(chr.data[,c(15,17,19)]-chr.data[,c(3,5,7)], 1, mean.fun)
    seg_partial.chr.data.day12diff=tree.weighted(chr, Day12diff.mval, Day12diff.mval.weight, chr.data$pos, Day12diff.bval, Day12diff.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.day12diff, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Day12_0diff_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Day3diff.mval.sd = apply(chr.data[,c(10,12,14)] - chr.data[,c(4,6,8)], 1, sd.fun)
    Day3diff.mval.sd = Day3diff.mval.sd + summary(Day3diff.mval.sd)[2]
    Day3diff.mval.weight = 1/Day3diff.mval.sd
    Day3diff.mval.weight = Day3diff.mval.weight / exp(mean(log(Day3diff.mval.weight), na.rm=T))
    Day3diff.bval.sd = apply(chr.data[,c(9,11,13)] - chr.data[,c(3,5,7)], 1, sd.fun)
    Day3diff.bval.sd = Day3diff.bval.sd + summary(Day3diff.bval.sd)[2]
    Day3diff.bval.weight = 1/Day3diff.bval.sd
    Day3diff.bval.weight = Day3diff.bval.weight / exp(mean(log(Day3diff.bval.weight), na.rm=T))

    Day3diff.mval = apply(chr.data[,c(10,12,14)]-chr.data[,c(4,6,8)], 1, mean.fun)
    Day3diff.bval = apply(chr.data[,c(9,11,13)]-chr.data[,c(3,5,7)], 1, mean.fun)
    seg_partial.chr.data.day3diff=tree.weighted(chr, Day3diff.mval, Day3diff.mval.weight, chr.data$pos, Day3diff.bval, Day3diff.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.day3diff, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Day3_0diff_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Day12_3diff.mval.sd = apply(chr.data[,c(16,18,20)] - chr.data[,c(10,12,14)], 1, sd.fun)
    Day12_3diff.mval.sd = Day12_3diff.mval.sd + summary(Day12_3diff.mval.sd)[2]
    Day12_3diff.mval.weight = 1/Day12_3diff.mval.sd
    Day12_3diff.mval.weight = Day12_3diff.mval.weight / exp(mean(log(Day12_3diff.mval.weight), na.rm=T))
    Day12_3diff.bval.sd = apply(chr.data[,c(15,17,19)] - chr.data[,c(9,11,13)], 1, sd.fun)
    Day12_3diff.bval.sd = Day12_3diff.bval.sd + summary(Day12_3diff.bval.sd)[2]
    Day12_3diff.bval.weight = 1/Day12_3diff.bval.sd
    Day12_3diff.bval.weight = Day12_3diff.bval.weight / exp(mean(log(Day12_3diff.bval.weight), na.rm=T))

    Day12_3diff.mval = apply(chr.data[,c(16,18,20)]-chr.data[,c(10,12,14)], 1, mean.fun)
    Day12_3diff.bval = apply(chr.data[,c(15,17,19)]-chr.data[,c(9,11,13)], 1, mean.fun)
    seg_partial.chr.data.day12_3diff=tree.weighted(chr, Day12_3diff.mval, Day12_3diff.mval.weight, chr.data$pos, Day12_3diff.bval, Day12_3diff.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.day12_3diff, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Day12_3diff_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Set5diff.mval.sd = apply(chr.data[,c(6,12,18)] - chr.data[,c(4,10,16)], 1, sd.fun)
    Set5diff.mval.sd = Set5diff.mval.sd + summary(Set5diff.mval.sd)[2]
    Set5diff.mval.weight = 1/Set5diff.mval.sd
    Set5diff.mval.weight = Set5diff.mval.weight / exp(mean(log(Set5diff.mval.weight), na.rm=T))
    Set5diff.bval.sd = apply(chr.data[,c(5,11,17)] - chr.data[,c(3,9,15)], 1, sd.fun)
    Set5diff.bval.sd = Set5diff.bval.sd + summary(Set5diff.bval.sd)[2]
    Set5diff.bval.weight = 1/Set5diff.bval.sd
    Set5diff.bval.weight = Set5diff.bval.weight / exp(mean(log(Set5diff.bval.weight), na.rm=T))

    Set5diff.mval = apply(chr.data[,c(6,12,18)]-chr.data[,c(4,10,16)], 1, mean.fun)
    Set5diff.bval = apply(chr.data[,c(5,11,17)]-chr.data[,c(3,9,15)], 1, mean.fun)
    seg_partial.chr.data.set5diff=tree.weighted(chr, Set5diff.mval, Set5diff.mval.weight, chr.data$pos, Set5diff.bval, Set5diff.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.set5diff, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set5_4diff_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Set6diff.mval.sd = apply(chr.data[,c(8,14,20)] - chr.data[,c(4,10,16)], 1, sd.fun)
    Set6diff.mval.sd = Set6diff.mval.sd + summary(Set6diff.mval.sd)[2]
    Set6diff.mval.weight = 1/Set6diff.mval.sd
    Set6diff.mval.weight = Set6diff.mval.weight / exp(mean(log(Set6diff.mval.weight), na.rm=T))
    Set6diff.bval.sd = apply(chr.data[,c(7,10,19)] - chr.data[,c(3,9,15)], 1, sd.fun)
    Set6diff.bval.sd = Set6diff.bval.sd + summary(Set6diff.bval.sd)[2]
    Set6diff.bval.weight = 1/Set6diff.bval.sd
    Set6diff.bval.weight = Set6diff.bval.weight / exp(mean(log(Set6diff.bval.weight), na.rm=T))

    Set6diff.mval = apply(chr.data[,c(8,14,20)]-chr.data[,c(4,10,16)], 1, mean.fun)
    Set6diff.bval = apply(chr.data[,c(7,13,19)]-chr.data[,c(3,9,15)], 1, mean.fun)
    seg_partial.chr.data.Set6diff=tree.weighted(chr, Set6diff.mval, Set6diff.mval.weight, chr.data$pos, Set6diff.bval, Set6diff.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.Set6diff, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set6_4diff_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")

    Set6_5diff.mval.sd = apply(chr.data[,c(8,14,20)] - chr.data[,c(6,12,18)], 1, sd.fun)
    Set6_5diff.mval.sd = Set6_5diff.mval.sd + summary(Set6_5diff.mval.sd)[2]
    Set6_5diff.mval.weight = 1/Set6_5diff.mval.sd
    Set6_5diff.mval.weight = Set6_5diff.mval.weight / exp(mean(log(Set6_5diff.mval.weight), na.rm=T))
    Set6_5diff.bval.sd = apply(chr.data[,c(7,13,19)] - chr.data[,c(5,11,17)], 1, sd.fun)
    Set6_5diff.bval.sd = Set6_5diff.bval.sd + summary(Set6_5diff.bval.sd)[2]
    Set6_5diff.bval.weight = 1/Set6_5diff.bval.sd
    Set6_5diff.bval.weight = Set6_5diff.bval.weight / exp(mean(log(Set6_5diff.bval.weight), na.rm=T))

    Set6_5diff.mval = apply(chr.data[,c(8,14,20)]-chr.data[,c(6,12,18)], 1, mean.fun)
    Set6_5diff.bval = apply(chr.data[,c(7,13,19)]-chr.data[,c(5,11,17)], 1, mean.fun)
    seg_partial.chr.data.Set6_5diff=tree.weighted(chr, Set6_5diff.mval, Set6_5diff.mval.weight, chr.data$pos, Set6_5diff.bval, Set6_5diff.bval.weight, p.thresh=0.01, mincut=2, option=1)
    write.table(seg_partial.chr.data.Set6_5diff, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set6_5diff_Partialweighted_result_chr", chr, ".txt", sep=""), row.names=F, quote=F, sep="\t")
}

qc.fun = function(x, n=9) {return(sum(is.na(x)) + sum(x[c(1+(1:n)*4, 2+(1:n)*4)] > 0, na.rm=T))}

individual_Set456.fun = function(chr, filename, ord, p.thresh=1e-6, diff.thresh=0.3, mincut=5) {
    chr.data = read.table(filename, header=T)
    res = apply(chr.data, 1, qc.fun)
    chr.data = chr.data[which(res==0), ord]

    write.table(fun.chr(chr, chr.data[,4], chr.data$pos, chr.data[,3], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set4Day0_result_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
    write.table(fun.chr(chr, chr.data[,6], chr.data$pos, chr.data[,5], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set5Day0_result_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
    write.table(fun.chr(chr, chr.data[,8], chr.data$pos, chr.data[,7], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set6Day0_result_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
    write.table(fun.chr(chr, chr.data[,10], chr.data$pos, chr.data[,9], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set4Day3_result_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
    write.table(fun.chr(chr, chr.data[,12], chr.data$pos, chr.data[,11], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set5Day3_result_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
    write.table(fun.chr(chr, chr.data[,14], chr.data$pos, chr.data[,13], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set6Day3_result_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
    write.table(fun.chr(chr, chr.data[,16], chr.data$pos, chr.data[,15], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set4Day12_result_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
    write.table(fun.chr(chr, chr.data[,18], chr.data$pos, chr.data[,17], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set5Day12_result_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
    write.table(fun.chr(chr, chr.data[,20], chr.data$pos, chr.data[,19], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste("/nfs_exports/genomes/1/projects/WGBS/PCGP/BucketIntermediate/SJNORM/Segmentation/Set6Day12_result_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
}

individual.fun = function(chr, filename, ord=NULL, p.thresh=1e-6, diff.thresh=0.3, mincut=5, output.dir="") {
    chr.data = read.table(filename, header=T)
    if (!is.null(ord)) chr.data = chr.data[, ord]
    #res = apply(chr.data, 1, qc.fun, n=(ncol(chr.data) - 2) / 4)
    #chr.data = chr.data[which(res==0),]

    for (i in 1: ((ncol(chr.data) - 2) / 4)) {
	idx = which(chr.data[, i*4 + 1] == 0 & (chr.data[, i*4 + 2] == 0 | chr.data[, i*4 + 2] == 2))
	write.table(fun.chr(chr, chr.data[idx,i * 4], chr.data$pos[idx], chr.data[idx,i * 4 - 1], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste(output.dir, names(chr.data)[4 * i], "_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
	print(paste("Done: ", output.dir, names(chr.data)[4 * i], "_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt, Time: ", date(), sep=""))
    }
}

individual_SD.fun = function(chr, filename, ord=NULL, p.thresh=1e-6, diff.thresh=0.3, mincut=5, output.dir="") {
    chr.data = read.table(filename, header=T)
    if (!is.null(ord)) chr.data = chr.data[, ord]
    #res = apply(chr.data, 1, qc.fun, n=(ncol(chr.data) - 2) / 4)
    #chr.data = chr.data[which(res==0),]

    for (i in 1: ((ncol(chr.data) - 2) / 4)) {
	idx = which(chr.data[, i*4 + 1] == 0 & (chr.data[, i*4 + 2] == 0 | chr.data[, i*4 + 2] == 2))
	write.table(fun_SD.chr(chr, chr.data[idx,i * 4], chr.data$pos[idx], chr.data[idx,i * 4 - 1], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste(output.dir, names(chr.data)[4 * i], "_SD_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
	print(paste("Done: ", output.dir, names(chr.data)[4 * i], "_SD_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt, Time: ", date(), sep=""))
    }
}

individual_diff.fun = function(chr, filename, idx1, idx2, p.thresh=1e-6, diff.thresh=0.3, mincut=5, output.dir="") {
    chr.data = read.table(filename, header=T)

	idx = which(chr.data[, idx1*4 + 1] == 0 & (chr.data[, idx1*4 + 2] == 0 | chr.data[, idx1*4 + 2] == 2) & chr.data[, idx2*4 + 1] == 0 & (chr.data[, idx2*4 + 2] == 0 | chr.data[, idx2*4 + 2] == 2))
	write.table(fun.chr(chr, chr.data[idx,idx2 * 4] - chr.data[idx,idx1 * 4], chr.data$pos[idx], chr.data[idx,idx2 * 4 - 1]-chr.data[idx,idx1 * 4 - 1], p.thresh=p.thresh, diff.thresh=diff.thresh, mincut=mincut),
		paste(output.dir, names(chr.data)[4 * idx2], "_", names(chr.data)[4 * idx1], "_diff_chr", chr, "_p", round(log10(p.thresh)), "_d", round(diff.thresh*10), "_m", mincut, ".txt", sep=""), row.names=F, quote=F, sep="\t")
}

plotPMD = function(data, segment, loc.start, loc.end, filename, main = "") {
	png(filename, width=2500, height=1600)
	par(mfrow=c(6,1))
	par(cex=1, cex.axis=1.5, cex.lab=1.8, cex.main=2)
	break1 = (loc.start * 5 + loc.end) / 6
	break2 = (loc.start * 4 + 2 * loc.end) / 6
	break3 = (loc.start * 3 + 3 * loc.end) / 6
	break4 = (loc.start * 2 + 4 * loc.end) / 6
	break5 = (loc.start + 5 * loc.end) / 6
	plot(data$pos, data$beta, "p", pch=20, col="gray", xlim=c(loc.start, break1), ylim=c(-0.05, 1), xlab="Position", ylab="beta", main = main)
	idx = which((segment$start >= loc.start & segment$end <= break1) | (segment$start <= loc.start & segment$end >= loc.start)  | (segment$start <= break1 & segment$end >= break1))
	for (i in idx) {
		lines(c(segment$start[i], segment$end[i]), rep(-0.05, 2), lwd=2, col="blue")
	}
	plot(data$pos, data$beta, "p", pch=20, col="gray", xlim=c(break1, break2), ylim=c(-0.05, 1), xlab="Position", ylab="beta", main="")
	idx = which((segment$start >= break1 & segment$end <= break2) | (segment$start <= break1 & segment$end >= break1)  | (segment$start <= break2 & segment$end >= break2))
	for (i in idx) {
		lines(c(segment$start[i], segment$end[i]), rep(-0.05, 2), lwd=2, col="blue")
	}
	plot(data$pos, data$beta, "p", pch=20, col="gray", xlim=c(break2, break3), ylim=c(-0.05, 1), xlab="Position", ylab="beta", main="")
	idx = which((segment$start >= break2 & segment$end <= break3) | (segment$start <= break2 & segment$end >= break2)  | (segment$start <= break3 & segment$end >= break3))
	for (i in idx) {
		lines(c(segment$start[i], segment$end[i]), rep(-0.05, 2), lwd=2, col="blue")
	}
	plot(data$pos, data$beta, "p", pch=20, col="gray", xlim=c(break3, break4), ylim=c(-0.05, 1), xlab="Position", ylab="beta", main="")
	idx = which((segment$start >= break3 & segment$end <= break4) | (segment$start <= break3 & segment$end >= break3)  | (segment$start <= break4 & segment$end >= break4))
	for (i in idx) {
		lines(c(segment$start[i], segment$end[i]), rep(-0.05, 2), lwd=2, col="blue")
	}
	plot(data$pos, data$beta, "p", pch=20, col="gray", xlim=c(break4, break5), ylim=c(-0.05, 1), xlab="Position", ylab="beta", main="")
	idx = which((segment$start >= break4 & segment$end <= break5) | (segment$start <= break4 & segment$end >= break4)  | (segment$start <= break5 & segment$end >= break5))
	for (i in idx) {
		lines(c(segment$start[i], segment$end[i]), rep(-0.05, 2), lwd=2, col="blue")
	}
	plot(data$pos, data$beta, "p", pch=20, col="gray", xlim=c(break5, loc.end), ylim=c(-0.05, 1), xlab="Position", ylab="beta", main="")
	idx = which((segment$start >= break5 & segment$end <= loc.end) | (segment$start <= break5 & segment$end >= break5)  | (segment$start <= loc.end & segment$end >= loc.end))
	for (i in idx) {
		lines(c(segment$start[i], segment$end[i]), rep(-0.05, 2), lwd=2, col="blue")
	}
	dev.off()
}
#plotPMD(data, segmentation, 2e6, 3e6, "Set4Day12_PMD_2M-3M.png", main="Set4, Day12, chr1")
 

#for (i in 1:22) triplicate.diff(i, paste("/nfs_exports/genomes/1/PCGP/BucketIntermediate/SJNORM/Methylation/BisSNP/SJNORM_set456", i,  "summary.txt", sep="_"), ord=c(1:4,15,16,27,28,7,8,19,20,31,32,11,12,23,24,35,36))

#for (i in 1:22) individual.fun(i, paste("/nfs_exports/genomes/1/projects/WGBS/PCGP2/BucketIntermediate/SJBALL/SJNALM6", i,  "summary.txt", sep="_"), diff.thresh=0.1, mincut=5, output.dir = "/nfs_exports/genomes/1/projects/WGBS/PCGP2/BucketIntermediate/SJBALL/Segmentation/")

### Usage: Rscript WGBS_Segment.fun.mod.R [summary/dir/] [prefix] [output/dir/] [p.thresh|1e-6]
argvs   <- commandArgs(TRUE)
sumDIR  <- argvs[1]
PREFIX  <- argvs[2]
OUTDIR  <- argvs[3]
PTHRESH <- as.numeric(argvs[4])
for (i in 1:22) individual_SD.fun(i, paste(sumDIR,PREFIX,"_",i,"_summary.txt", sep=""), diff.thresh=0.1, mincut=5, output.dir=OUTDIR, p.thresh=PTHRESH)
